@@ -8,26 +8,26 @@ Task related commands.
 """
 import os
 import readline
-from datetime import datetime, date, timedelta
+from datetime import datetime, timedelta
 from dateutil import rrule
-from sqlobject import SQLObjectNotFound, LIKE, AND, OR
+from sqlobject import LIKE, AND, OR, NOT
+from sqlobject.sqlbuilder import LEFTJOINOn
 
 from db import Config, Keyword, Project, Task, \
-               TaskKeyword, ProjectKeyword, Recurrence
+               TaskKeyword, Recurrence
 import bugutils
 import dbutils
-import dateutils
+import ydateutils
 import parseutils
 import tui
-from completers import ProjectCompleter, projectAndKeywordCompleter,\
+from completers import ProjectCompleter, projectAndKeywordCompleter, \
                        taskIdCompleter, recurrenceCompleter, dueDateCompleter
-from yokadiexception import YokadiException
+from yokadiexception import YokadiException, BadUsageException
 from textlistrenderer import TextListRenderer
 from xmllistrenderer import XmlListRenderer
 from csvlistrenderer import CsvListRenderer
 from htmllistrenderer import HtmlListRenderer
 from plainlistrenderer import PlainListRenderer
-
 from yokadioptionparser import YokadiOptionParser
 
 gRendererClassDict = dict(
@@ -38,61 +38,82 @@ gRendererClassDict = dict(
     plain=PlainListRenderer,
     )
 
+NOTE_KEYWORD = "_note"
+
 class TaskCmd(object):
     def __init__(self):
         self.lastTaskId = None
+        self.kFilters = [] # Permanent keyword filters (List of KeywordFilter)
+        self.pFilter = ""  # Permanent project filter (name of project)
         for name in bugutils.PROPERTY_NAMES:
             dbutils.getOrCreateKeyword(name, interactive=False)
+        dbutils.getOrCreateKeyword(NOTE_KEYWORD, interactive=False)
+
+    def _t_add(self, cmd, line):
+        """Code shared by t_add, bug_add and n_add."""
+        line = line.strip()
+        if not line:
+            raise BadUsageException("Missing parameters")
+        projectName, title, keywordDict = parseutils.parseLine(line)
+        if not title:
+            raise BadUsageException("Missing title")
+        task = dbutils.addTask(projectName, title, keywordDict)
+        if not task:
+            tui.reinjectInRawInput(u"%s %s" % (cmd, line))
+            return None
+        self.lastTaskId = task.id
+        return task
+
+    def do_t_add(self, line):
+        """Add new task. Will prompt to create keywords if they do not exist.
+        t_add <projectName> [@<keyword1>] [@<keyword2>] <title>"""
+        task = self._t_add("t_add", line)
+        if task:
+            print "Added task '%s' (id=%d)" % (task.title, task.id)
+    complete_t_add = projectAndKeywordCompleter
 
     def do_bug_add(self, line):
         """Add a bug-type task. Will create a task and ask additional info.
-        bug_add <project_name> [@<keyword1>] [@<keyword2>] <Bug description>
+        bug_add <project_name> [@<keyword1>] [@<keyword2>] <title>
         """
-        projectName, title, keywordDict = parseutils.parseLine(line)
-
-        task = dbutils.addTask(projectName, title, keywordDict)
+        task = self._t_add("bug_add", line)
         if not task:
-            tui.reinjectInRawInput(u"bug_add " + line)
             return
 
+        keywordDict = task.getKeywordDict()
         bugutils.editBugKeywords(keywordDict)
         task.setKeywordDict(keywordDict)
-
         task.urgency = bugutils.computeUrgency(keywordDict)
-        self.lastTaskId = task.id
 
-        print "Added bug '%s' (id=%d, urgency=%d)" % (title, task.id, task.urgency)
+        print "Added bug '%s' (id=%d, urgency=%d)" % (task.title, task.id, task.urgency)
 
     complete_bug_add = ProjectCompleter(1)
 
+    def do_n_add(self, line):
+        """Add a note. A note is a task with the @_note keyword.
+        n_add <project_name> [@<keyword1>] [@<keyword2>] <title>
+        """
+        task = self._t_add("n_add", line)
+        if not task:
+            return
+        keywordDict = task.getKeywordDict()
+        keywordDict[NOTE_KEYWORD] = None
+        task.setKeywordDict(keywordDict)
+        print "Added note '%s' (id=%d)" % (task.title, task.id)
+    complete_n_add = projectAndKeywordCompleter
+
     def do_bug_edit(self, line):
         """Edit a bug.
         bug_edit <id>"""
-        task = self.getTaskFromId(line)
+        task = self._t_edit(line)
+        if not task:
+            return
 
-        # Create task line
-        taskLine = parseutils.createLine("", task.title, task.getKeywordDict())
-
-        # Edit
-        while True:
-            print "(Press Ctrl+C to cancel)"
-            try:
-                line = tui.editLine(taskLine)
-                if not line.strip():
-                    tui.warning("Indicate a bug title !")
-                    continue
-            except KeyboardInterrupt:
-                print
-                print "Cancelled"
-                return
-            foo, title, keywordDict = parseutils.parseLine(task.project.name+" "+line)
-            if dbutils.updateTask(task, task.project.name, title, keywordDict):
-                break
-        editBugKeywords(keywordDict)
+        keywordDict = task.getKeywordDict()
+        bugutils.editBugKeywords(keywordDict)
         task.setKeywordDict(keywordDict)
-
-        # Update bug
         task.urgency = bugutils.computeUrgency(keywordDict)
+    complete_bug_edit = taskIdCompleter
 
     def getTaskFromId(self, line):
         line = line.strip()
@@ -105,29 +126,10 @@ class TaskCmd(object):
             self.lastTaskId = task.id
         return task
 
-    def do_t_add(self, line):
-        """Add new task. Will prompt to create keywords if they do not exist.
-        t_add <projectName> [@<keyword1>] [@<keyword2>] <Task description>"""
-        if not line:
-            print "Give at least a task name !"
-            return
-        projectName, title, keywordDict = parseutils.parseLine(line)
-        if not title:
-            raise YokadiException("You should give a task title")
-        task = dbutils.addTask(projectName, title, keywordDict)
-        if task:
-            print "Added task '%s' (id=%d)" % (title, task.id)
-        else:
-            tui.reinjectInRawInput(u"t_add " + line)
-        if task:
-            self.lastTaskId = task.id
-
-    complete_t_add = projectAndKeywordCompleter
-
     def do_t_describe(self, line):
         """Starts an editor to enter a longer description of a task.
         t_describe <id>"""
-        task=self.getTaskFromId(line)
+        task = self.getTaskFromId(line)
         try:
             description = tui.editText(task.description)
         except Exception, e:
@@ -144,23 +146,24 @@ class TaskCmd(object):
     def do_t_urgency(self, line):
         """Defines urgency of a task.
         t_urgency <id> <value>"""
-        tokens = line.split(" ")
-        if len(tokens)!=2:
-            raise YokadiException("You must provide a taskId and an urgency value") 
+
+        tokens = parseutils.simplifySpaces(line).split(" ")
+        if len(tokens) != 2:
+            raise BadUsageException("You must provide a taskId and an urgency value")
         task = self.getTaskFromId(tokens[0])
         try:
             # Do not use isdigit(), so that we can set negative urgency. This
             # make it possible to stick tasks to the bottom of the list.
             urgency = int(tokens[1])
         except ValueError:
-            raise YokadiException("Task urgency must be a digit")
+            raise BadUsageException("Task urgency must be a digit")
 
-        if urgency>100:
+        if urgency > 100:
             tui.warning("Max urgency is 100")
-            urgency=100
-        elif urgency<-99:
+            urgency = 100
+        elif urgency < -99:
             tui.warning("Min urgency is -99")
-            urgency=-99
+            urgency = -99
 
         task.urgency = urgency
 
@@ -189,7 +192,7 @@ class TaskCmd(object):
     complete_t_mark_new = taskIdCompleter
 
     def _t_set_status(self, line, status):
-        task=self.getTaskFromId(line)
+        task = self.getTaskFromId(line)
         if task.recurrence and status == "done":
             task.dueDate = task.recurrence.getNext(task.dueDate)
             print "Task '%s' next occurrence is scheduled at %s" % (task.title, task.dueDate)
@@ -206,8 +209,8 @@ class TaskCmd(object):
         """Apply a command to several tasks.
         t_apply <id1>[,<id2>,[<id3>]...]] <command> <args>"""
         tokens = line.split(" ", 2)
-        if len(tokens)<2:
-            raise YokadiException("Give at least a task id and a command. See 'help t_apply'")
+        if len(tokens) < 2:
+            raise BadUsageException("Give at least a task id and a command")
         idStringList = tokens[0]
         cmd = tokens[1]
         if len(tokens) == 3:
@@ -232,7 +235,7 @@ class TaskCmd(object):
     def do_t_remove(self, line):
         parser = self.parser_t_remove()
         options, args = parser.parse_args(line)
-        task=self.getTaskFromId(' '.join(args))
+        task = self.getTaskFromId(' '.join(args))
         if not options.force:
             if not tui.confirm("Remove task '%s'" % task.title):
                 return
@@ -260,11 +263,11 @@ class TaskCmd(object):
     def do_t_purge(self, line):
         parser = self.parser_t_purge()
         options, args = parser.parse_args(line)
-        filters=[]
-        filters.append(Task.q.status=="done")
-        filters.append(Task.q.doneDate<(datetime.now()-timedelta(days=options.delay)))
-        tasks=Task.select(AND(*filters))
-        if tasks.count()==0:
+        filters = []
+        filters.append(Task.q.status == "done")
+        filters.append(Task.q.doneDate < (datetime.now() - timedelta(days=options.delay)))
+        tasks = Task.select(AND(*filters))
+        if tasks.count() == 0:
             print "No tasks need to be purged"
             return
         print "The following tasks will be removed:"
@@ -285,34 +288,52 @@ class TaskCmd(object):
             "Keyword filtering is achieved with '@'. Ex.: "
             "t_list @home, t_list @_bug=2394")
 
-        parser.add_option("-a", "--all", dest="all",
-                          default=False, action="store_true",
+        parser.add_option("-a", "--all", dest="status",
+                          action="store_const", const="all",
                           help="all tasks (done and to be done)")
 
+        parser.add_option("--started", dest="status",
+                          action="store_const", const="started",
+                          help="only started tasks")
+
         rangeList = ["today", "thisweek", "thismonth", "all"]
         parser.add_option("-d", "--done", dest="done",
                           help="only done tasks. <range> must be one of %s" % ", ".join(rangeList),
                           metavar="<range>")
 
-        parser.add_option("-u", "--top-urgent", dest="topUrgent",
-                          default=False, action="store_true",
-                          help="top 5 urgent tasks of each project based on urgency")
+        parser.add_option("-u", "--urgency", dest="urgency",
+                          type="int",
+                          help="tasks with urgency greater or equal than <urgency>",
+                          metavar="<urgency>")
 
         parser.add_option("-t", "--top-due", dest="topDue",
                           default=False, action="store_true",
                           help="top 5 urgent tasks of each project based on due date")
 
-        parser.add_option("--overdue", dest="overdue",
-                          default=False, action="store_true",
+        parser.add_option("--overdue", dest="due",
+                          action="append_const", const="now",
                           help="all overdue tasks")
 
+        parser.add_option("--due", dest="due",
+                          action="append",
+                          help="""only list tasks due before/after <limit>. <limit> is a
+                          date optionaly prefixed with a comparison operator.
+                          Valid operators are: <, <=, >=, and >.
+                          Example of valid limits:
+
+                          - tomorrow: due date <= tomorrow, 23:59:59
+                          - today: due date <= today, 23:59:59
+                          - >today: due date > today: 23:59:59
+                          """,
+                          metavar="<limit>")
+
         parser.add_option("-k", "--keyword", dest="keyword",
                           help="Group tasks by given keyword instead of project. The % wildcard can be used.",
                           metavar="<keyword>")
 
         parser.add_option("-s", "--search", dest="search",
                           action="append",
-                          help="only list tasks which title or description match <value>. You can repeat this option to search on multiple words.",
+                          help="only list tasks whose title or description match <value>. You can repeat this option to search on multiple words.",
                           metavar="<value>")
 
         formatList = ["auto"] + gRendererClassDict.keys()
@@ -325,63 +346,101 @@ class TaskCmd(object):
                           metavar="<file>")
         return parser
 
-    def do_t_list(self, line):
-        doneRangeList= ["today", "thisweek", "thismonth"]
+    def _parseListLine(self, parser, line):
+        """
+        Parse line with parser, returns a tuple of the form
+        (options, projectList, filters)
+        """
+        options, args = parser.parse_args(line)
+        if len(args) > 0:
+            projectName, keywordFilters = parseutils.extractKeywords(u" ".join(args))
+        else:
+            projectName = ""
+            keywordFilters = []
 
-        def keywordDictIsSubsetOf(taskKeywordDict, wantedKeywordDict):
-            # Returns true if taskKeywordDict is a subset of wantedKeywordDict
-            # taskKeywordDict is considered a subset of wantedKeywordDict if:
-            # 1. All wantedKeywordDict keys are in taskKeywordDict
-            # 2. All wantedKeywordDict valued keywords have the same value
-            #    in taskKeywordDict
-            for wantedKeyword, wantedValue in wantedKeywordDict.items():
-                if not wantedKeyword in taskKeywordDict:
-                    return False
-                if wantedValue and taskKeywordDict[wantedKeyword] != wantedValue:
-                    return False
-            return True
+        if self.kFilters:
+            # Add keyword filter
+            keywordFilters.extend(self.kFilters)
 
-        def taskHasWantedKeywordDict(task, wantedKeywordDict):
-            """
-            @param task: task object
-            @param wantedKeywordDict: dict of name/value of wanted keyword
-            # a task is considered a subset of wantedKeywordDict if:
-            # 1. All wantedKeywordDict keys are in task or project keywords
-            # 2. All wantedKeywordDict valued keywords have the same value
-            #    in task or project keyword"""
-            for wantedKeyword, wantedValue in wantedKeywordDict.items():
-                taskFilters=[Task.q.id==task.id,
-                         TaskKeyword.q.taskID==task.id,
-                         TaskKeyword.q.keywordID==Keyword.q.id,
-                         LIKE(Keyword.q.name, wantedKeyword)]
-
-                projectFilters=[Project.q.id==task.projectID,
-                                ProjectKeyword.q.projectID==Project.q.id,
-                                ProjectKeyword.q.keyword==Keyword.q.id,
-                                LIKE(Keyword.q.name, wantedKeyword)]
-
-                if wantedValue:
-                    taskFilters.append(TaskKeyword.q.value==wantedValue)
-                    projectFilters.append(ProjectKeyword.q.value==wantedValue)
-
-                if Task.select(AND(*taskFilters)).count()==0 and Task.select(AND(*projectFilters)).count()==0:
-                    return False
-            # All critera were met, return ok
-            return True
-
-        def createFilterFromRange(_range):
-            # Parse the _range string and return an SQLObject filter
-            minDate = date.today()
-            if _range == "today":
-                pass
-            elif _range == "thisweek":
-                minDate -= timedelta(minDate.weekday())
-            elif _range == "thismonth":
-                minDate = minDate.replace(day = 1)
+        if not projectName:
+            if self.pFilter:
+                # If a project filter is defined, use it as none was provided
+                projectName = self.pFilter
             else:
-                raise YokadiException("Invalid range value '%s'" % _range)
+                # Take all project if none provided
+                projectName = "%"
 
-            return Task.q.doneDate>=minDate
+        if projectName.startswith("!"):
+            projectName = projectName[1:]
+            projectList = Project.select(NOT(LIKE(Project.q.name, projectName)))
+        else:
+            projectList = Project.select(LIKE(Project.q.name, projectName))
+
+        if projectList.count() == 0:
+            raise YokadiException("Found no project matching '%s'" % projectName)
+
+        # Check keywords exist
+        parseutils.warnIfKeywordDoesNotExist(keywordFilters)
+
+        # Filtering and sorting according to parameters
+        filters = []
+
+        # Filter on keywords
+        for keywordFilter in keywordFilters:
+            filters.append(keywordFilter.filter())
+
+        # Search
+        if options.search:
+            for word in options.search:
+                if word.startswith("@"):
+                    tui.warning("Maybe you want keyword search (without -s option) "
+                                "instead of plain text search?")
+                filters.append(OR(LIKE(Task.q.title, "%" + word + "%"),
+                                  LIKE(Task.q.description, "%" + word + "%")))
+
+        return options, projectList, filters
+
+    def _renderList(self, renderer, projectList, filters, order, limit, groupKeyword):
+        """
+        Render a list using renderer, according to the restrictions set by the
+        other parameters
+        """
+        if groupKeyword:
+            if groupKeyword.startswith("@"):
+                groupKeyword = groupKeyword[1:]
+            for keyword in Keyword.select(LIKE(Keyword.q.name, groupKeyword)):
+                if unicode(keyword.name).startswith("_") and not groupKeyword.startswith("_"):
+                    #BUG: cannot filter on db side because sqlobject does not understand ESCAPE needed whith _
+                    continue
+                taskList = Task.select(AND(TaskKeyword.q.keywordID == keyword.id,
+                                           *filters),
+                                       orderBy=order, limit=limit, distinct=True,
+                                       join=LEFTJOINOn(Task, TaskKeyword, Task.q.id == TaskKeyword.q.taskID))
+                taskList = list(taskList)
+                if projectList:
+                    taskList = [x for x in taskList if x.project in projectList]
+                if len(taskList) > 0:
+                    renderer.addTaskList(unicode(keyword), taskList)
+            renderer.end()
+        else:
+            hiddenProjectNames = []
+            for project in projectList:
+                if not project.active:
+                    hiddenProjectNames.append(project.name)
+                    continue
+                taskList = Task.select(AND(Task.q.projectID == project.id, *filters),
+                                       orderBy=order, limit=limit, distinct=True,
+                                       join=LEFTJOINOn(Task, TaskKeyword, Task.q.id == TaskKeyword.q.taskID))
+                taskList = list(taskList)
+
+                if len(taskList) > 0:
+                    renderer.addTaskList(unicode(project), taskList)
+            renderer.end()
+
+            if len(hiddenProjectNames) > 0:
+                tui.info("hidden projects: %s" % ", ".join(hiddenProjectNames))
+
+    def do_t_list(self, line):
 
         def selectRendererClass():
             if options.format != "auto":
@@ -398,53 +457,36 @@ class TaskCmd(object):
             return gRendererClassDict.get(ext[1:], defaultRendererClass)
 
         #BUG: completion based on parameter position is broken when parameter is given
-        parser = self.parser_t_list()
-        options, args = parser.parse_args(line)
-        if len(args) > 0:
-            projectName, keywordDict = parseutils.extractKeywords(u" ".join(args))
-        else:
-            projectName = ""
-            keywordDict = {}
+        options, projectList, filters = self._parseListLine(self.parser_t_list(), line)
 
-        if not projectName:
-            # Take all project if none provided
-            projectName="%"
+        # Skip notes
+        filters.append(parseutils.KeywordFilter("!@" + NOTE_KEYWORD).filter())
 
-        projectList = Project.select(LIKE(Project.q.name, projectName))
-
-        if projectList.count()==0:
-            tui.error("Found no project matching '%s'" % projectName)
-            return
-
-        # Check keywords exist
-        for keyword in keywordDict.keys():
-            if Keyword.select(LIKE(Keyword.q.name, keyword)).count()==0:
-                tui.error("Keyword %s is unknown." % keyword)
-
-        # Filtering and sorting according to parameters
-        filters=[]
-        order=-Task.q.urgency, Task.q.creationDate
-        limit=None
+        # Handle t_list specific options
+        order = -Task.q.urgency, Task.q.creationDate
+        limit = None
         if options.done:
-            filters.append(Task.q.status=='done')
+            filters.append(Task.q.status == 'done')
             if options.done != "all":
-                filters.append(createFilterFromRange(options.done))
-        elif not options.all:
-            filters.append(Task.q.status!='done')
-        if options.topUrgent:
-            order=-Task.q.urgency
-            limit=5
+                filters.append(parseutils.createFilterFromRange(options.done))
+        elif options.status == "all":
+            pass
+        elif options.status == "started":
+            filters.append(Task.q.status == 'started')
+        else:
+            filters.append(Task.q.status != 'done')
+        if options.urgency:
+            order = -Task.q.urgency
+            filters.append(Task.q.urgency >= options.urgency)
         if options.topDue:
-            filters.append(Task.q.dueDate!=None)
-            order=Task.q.dueDate
-            limit=5
-        if options.overdue:
-            filters.append(Task.q.dueDate<datetime.now())
-            order=Task.q.dueDate
-        if options.search:
-            for word in options.search:
-                filters.append(OR(LIKE(Task.q.title, "%"+word+"%"),
-                                  LIKE(Task.q.description, "%"+word+"%")))
+            filters.append(Task.q.dueDate != None)
+            order = Task.q.dueDate
+            limit = 5
+        if options.due:
+            for due in options.due:
+                dueOperator, dueLimit = ydateutils.parseDateLimit(due)
+                filters.append(dueOperator(Task.q.dueDate, dueLimit))
+            order = Task.q.dueDate
 
         # Define output
         if options.output:
@@ -457,54 +499,37 @@ class TaskCmd(object):
         renderer = rendererClass(out)
 
         # Fill the renderer
-        if options.keyword:
-            if options.keyword.startswith("@"):
-                options.keyword = options.keyword[1:]
-            for keyword in Keyword.select(LIKE(Keyword.q.name, options.keyword)):
-                if unicode(keyword.name).startswith("_") and not options.keyword.startswith("_"):
-                    #BUG: cannot filter on db side because sqlobject does not understand ESCAPE needed whith _
-                    continue
-                taskList = Task.select(AND(TaskKeyword.q.taskID == Task.q.id,
-                                           TaskKeyword.q.keywordID == keyword.id,
-                                           *filters),
-                                        orderBy=order, limit=limit)
-                taskList = list(taskList)
-                if keywordDict:
-                    # FIXME: factorize (see project oriented rendering below)
-                    taskList = [x for x in taskList if taskHasWantedKeywordDict(x, keywordDict)]
-                if projectList:
-                    taskList = [x for x in taskList if x.project in projectList]
-                if len(taskList) == 0:
-                    continue
-
-                renderer.addTaskList(unicode(keyword), taskList)
-            # Call renderer
-            renderer.end()
-        else:
-            hiddenProjectNames = []
-            for project in projectList:
-                if not project.active:
-                    hiddenProjectNames.append(project.name)
-                    continue
-                taskList = Task.select(AND(Task.q.projectID == project.id, *filters),
-                                       orderBy=order, limit=limit)
-
-                if keywordDict:
-                    taskList = [x for x in taskList if taskHasWantedKeywordDict(x, keywordDict)]
-                else:
-                    taskList = list(taskList)
-    
-                if len(taskList) == 0:
-                    continue
-
-                renderer.addTaskList(unicode(project), taskList)
-            renderer.end()
-    
-            if len(hiddenProjectNames) > 0:
-                tui.info("hidden projects: %s" % ", ".join(hiddenProjectNames))
-
+        self._renderList(renderer, projectList, filters, order, limit, options.keyword)
     complete_t_list = projectAndKeywordCompleter
 
+    def parser_n_list(self):
+        parser = YokadiOptionParser()
+        parser.set_usage("n_list [options] <project_or_keyword_filter>")
+        parser.set_description(
+            "List notes filtered by project and/or keywords. "
+            "'%' can be used as a wildcard in the project name: "
+            "to list projects starting with 'foo', use 'foo%'. "
+            "Keyword filtering is achieved with '@'. Ex.: "
+            "n_list @home, n_list @_bug=2394")
+
+        parser.add_option("-s", "--search", dest="search",
+                          action="append",
+                          help="only list notes whose title or description match <value>. You can repeat this option to search on multiple words.",
+                          metavar="<value>")
+
+        parser.add_option("-k", "--keyword", dest="keyword",
+                          help="Group tasks by given keyword instead of project. The % wildcard can be used.",
+                          metavar="<keyword>")
+        return parser
+
+    def do_n_list(self, line):
+        options, projectList, filters = self._parseListLine(self.parser_n_list(), line)
+        filters.append(parseutils.KeywordFilter("@" + NOTE_KEYWORD).filter())
+        order = Task.q.creationDate
+        renderer = TextListRenderer(tui.stdout)
+        self._renderList(renderer, projectList, filters, order, limit=None, groupKeyword=options.keyword)
+    complete_n_list = projectAndKeywordCompleter
+
     def do_t_reorder(self, line):
         """Reorder tasks of a project.
         It works by starting an editor with the task list: you can then change
@@ -513,8 +538,8 @@ class TaskCmd(object):
         t_reorder <project_name>"""
         project = Project.byName(line)
         taskList = Task.select(AND(Task.q.projectID == project.id,
-                                   Task.q.status    != 'done'),
-                               orderBy=-Task.q.urgency)
+                                   Task.q.status != 'done'),
+                               orderBy= -Task.q.urgency)
         lines = [ "%d,%s" % (x.id, x.title) for x in taskList]
         text = tui.editText("\n".join(lines))
 
@@ -550,7 +575,7 @@ class TaskCmd(object):
         parser = self.parser_t_show()
         options, args = parser.parse_args(line)
 
-        task=self.getTaskFromId(' '.join(args))
+        task = self.getTaskFromId(' '.join(args))
 
         if options.output in ("all", "summary"):
             keywordDict = task.getKeywordDict()
@@ -585,10 +610,8 @@ class TaskCmd(object):
 
     complete_t_show = taskIdCompleter
 
-    def do_t_edit(self, line):
-        """Edit a task.
-        t_edit <id>"""
-
+    def _t_edit(self, line):
+        """Code shared by t_edit and bug_edit."""
         def editComplete(text, state):
             """ Specific completer for the edit prompt.
             This subfunction should stay here because it needs to access to cmd members"""
@@ -598,7 +621,7 @@ class TaskCmd(object):
                 stripped = len(origline) - len(line)
                 begidx = readline.get_begidx() - stripped
                 endidx = readline.get_endidx() - stripped
-                if begidx>0:
+                if begidx > 0:
                     self.completion_matches = projectAndKeywordCompleter("", text, line, begidx, endidx, shift=1)
                 else:
                     self.completion_matches = []
@@ -612,8 +635,8 @@ class TaskCmd(object):
         # Create task line
         taskLine = parseutils.createLine("", task.title, task.getKeywordDict())
 
-        old_completer = readline.get_completer() # Backup previous completer to restore it in the end
-        readline.set_completer(editComplete)     # Switch to specific completer
+        oldCompleter = readline.get_completer() # Backup previous completer to restore it in the end
+        readline.set_completer(editComplete)    # Switch to specific completer
 
         while True:
             # Edit
@@ -621,19 +644,24 @@ class TaskCmd(object):
             try:
                 line = tui.editLine(taskLine)
                 if not line.strip():
-                    tui.warning("Indicate a task title !")
+                    tui.warning("Missing title")
                     continue
             except KeyboardInterrupt:
                 print
                 print "Cancelled"
-                readline.set_completer(old_completer)   # Restore standard completer
-                return
-            foo, title, keywordDict = parseutils.parseLine(task.project.name+" "+line)
+                task = None
+                break
+            foo, title, keywordDict = parseutils.parseLine(task.project.name + " " + line)
             if dbutils.updateTask(task, task.project.name, title, keywordDict):
                 break
 
-        readline.set_completer(old_completer)   # Restore standard completer
+        readline.set_completer(oldCompleter)   # Restore standard completer
+        return task
 
+    def do_t_edit(self, line):
+        """Edit a task.
+        t_edit <id>"""
+        self._t_edit(line)
     complete_t_edit = taskIdCompleter
 
     def do_t_set_project(self, line):
@@ -645,10 +673,10 @@ class TaskCmd(object):
     def do_t_project(self, line):
         """Set task's project.
         t_project <id> <project>"""
-        tokens = line.split(" ")
-        if len(tokens)!=2:
+        tokens = parseutils.simplifySpaces(line).split(" ")
+        if len(tokens) != 2:
             raise YokadiException("You should give two arguments: <task id> <project>")
-        task=self.getTaskFromId(tokens[0])
+        task = self.getTaskFromId(tokens[0])
         projectName = tokens[1]
 
         task.project = dbutils.getOrCreateProject(projectName)
@@ -686,16 +714,17 @@ class TaskCmd(object):
         - 12:               on the 12th of current month
 
         To reset a due date, use "none"."""
-        if len(line.split())<2:
+        line = parseutils.simplifySpaces(line)
+        if len(line.split()) < 2:
             raise YokadiException("Give a task id and time, date or date & time")
-        taskId, line=line.strip().split(" ", 1)
-        task=self.getTaskFromId(taskId)
+        taskId, line = line.strip().split(" ", 1)
+        task = self.getTaskFromId(taskId)
 
-        if line.lower()=="none":
-            task.dueDate=None
+        if line.lower() == "none":
+            task.dueDate = None
             print "Due date for task '%s' reset" % task.title
         else:
-            task.dueDate = dateutils.parseHumaneDateTime(line)
+            task.dueDate = ydateutils.parseHumaneDateTime(line)
             print "Due date for task '%s' set to %s" % (task.title, task.dueDate.ctime())
 
     complete_t_set_due = dueDateCompleter
@@ -705,11 +734,12 @@ class TaskCmd(object):
         """Add keywords to an existing task
         t_add_keyword <id> <@keyword1> <@keyword2>[=<value>]...
         """
-        tokens = line.split(" ", 1)
+        tokens = parseutils.simplifySpaces(line).split(" ", 1)
         if len(tokens) < 2:
             raise YokadiException("You should give at least two arguments: <task id> <keyword>")
-        task = self.getTaskFromId(tokens[0])
-        garbage, newKwDict = parseutils.extractKeywords(tokens[1])
+        task = dbutils.getTaskFromId(tokens[0])
+        garbage, keywordFilters = parseutils.extractKeywords(tokens[1])
+        newKwDict = parseutils.keywordFiltersToDict(keywordFilters)
         if garbage:
             raise YokadiException("Cannot parse line, got garbage (%s). Maybe you forgot to add @ before keyword ?"
                                    % garbage)
@@ -730,11 +760,11 @@ class TaskCmd(object):
         t_recurs <id> weekly <mo, tu, we, th, fr, sa, su> <hh:mm>
         t_recurs <id> daily <HH:MM>
         t_recurs <id> none (remove recurrence)"""
-        tokens = line.split()
+        tokens = parseutils.simplifySpaces(line).split()
         if len(tokens) < 2:
             raise YokadiException("You should give at least two arguments: <task id> <recurrence>")
         task = self.getTaskFromId(tokens[0])
-        
+
         # Define recurrence:
         freq = byminute = byhour = byweekday = bymonthday = bymonth = None
 
@@ -749,37 +779,37 @@ class TaskCmd(object):
             if len(tokens) != 3:
                 raise YokadiException("You should give time for daily task")
             freq = rrule.DAILY
-            byhour, byminute = dateutils.getHourAndMinute(tokens[2])
+            byhour, byminute = ydateutils.getHourAndMinute(tokens[2])
         elif tokens[1] == "weekly":
             freq = rrule.WEEKLY
             if len(tokens) != 4:
                 raise YokadiException("You should give day and time for weekly task")
-            byweekday = dateutils.getWeekDayNumberFromDay(tokens[2].lower())
-            byhour, byminute = dateutils.getHourAndMinute(tokens[3])
-        elif tokens[1] in ("monthly","quarterly"):
+            byweekday = ydateutils.getWeekDayNumberFromDay(tokens[2].lower())
+            byhour, byminute = ydateutils.getHourAndMinute(tokens[3])
+        elif tokens[1] in ("monthly", "quarterly"):
             if tokens[1] == "monthly":
                 freq = rrule.MONTHLY
             else:
                 # quarterly
                 freq = rrule.YEARLY
-                bymonth = [1,4,7,10]
+                bymonth = [1, 4, 7, 10]
             if len(tokens) < 4:
                 raise YokadiException("You should give day and time for %s task" % (tokens[1],))
             try:
                 bymonthday = int(tokens[2])
-                byhour, byminute = dateutils.getHourAndMinute(tokens[3])
+                byhour, byminute = ydateutils.getHourAndMinute(tokens[3])
             except ValueError:
-                POSITION = { "first" : 1, "second" : 2, "third" : 3, "fourth" : 4, "last" : -1 }
+                POSITION = { "first" : 1, "second" : 2, "third" : 3, "fourth" : 4, "last" :-1 }
                 if tokens[2].lower() in POSITION.keys() and len(tokens) == 5:
-                    byweekday = rrule.weekday(dateutils.getWeekDayNumberFromDay(tokens[3].lower()),
+                    byweekday = rrule.weekday(ydateutils.getWeekDayNumberFromDay(tokens[3].lower()),
                                               POSITION[tokens[2]])
-                    byhour, byminute = dateutils.getHourAndMinute(tokens[4])
+                    byhour, byminute = ydateutils.getHourAndMinute(tokens[4])
                     bymonthday = None # Default to current day number - need to be blanked                    
                 else:
                     raise YokadiException("Unable to understand date. See help t_recurs for details")
         elif tokens[1] == "yearly":
             freq = rrule.YEARLY
-            rDate = dateutils.parseHumaneDateTime(" ".join(tokens[2:]))
+            rDate = ydateutils.parseHumaneDateTime(" ".join(tokens[2:]))
             bymonth = rDate.month
             bymonthday = rDate.day
             byhour = rDate.hour
@@ -794,4 +824,32 @@ class TaskCmd(object):
         task.recurrence.setRrule(rr)
         task.dueDate = task.recurrence.getNext()
     complete_t_recurs = recurrenceCompleter
+
+
+    def do_t_filter(self, line):
+        """Define permanent keyword filter used by t_list
+        Ex.:
+            - t_filter @work (filter all task that have the "work" keyword)
+            - t_filter none (remove filter)"""
+        #TODO: add completion
+
+        if not line:
+            raise YokadiException("You must give keyword as argument or 'none' to reset filter")
+
+        if parseutils.simplifySpaces(line).lower() == "none":
+            self.kFilters = []
+            self.pFilter = ""
+            self.prompt = "yokadi> "
+        else:
+            projectName, keywordFilters = parseutils.extractKeywords(line)
+            self.kFilters = keywordFilters
+            self.pFilter = projectName
+            prompt = "y"
+            if self.pFilter:
+                prompt += " %s" % projectName
+            if self.kFilters:
+                parseutils.warnIfKeywordDoesNotExist(self.kFilters)
+                prompt += " %s" % (" ".join([str(k) for k in keywordFilters]))
+            self.prompt = "%s> " % prompt
+
 # vi: ts=4 sw=4 et
